#include <stdio.h>
#include <sys/types.h>
#include "windows.h"

#include "libusb.h"

#define NUM_ROBOTS 4

#define PAYLOAD_SIZE 6
#define PACKETS_SIZE PAYLOAD_SIZE*NUM_ROBOTS
// packet:
// -----------------------------------
// R | B | G | IR/flags | Right | Left
// -----------------------------------
// packet content:
// R, B, G: values from 0 (OFF) to 100 (ON max power)
// IR/flags:
// - first two bits are dedicated to the IRs:
// 0x00 => all IRs off
// 0x01 => back IR on
// 0x02 => front IRs on
// 0x03 => all IRs on
// - third bit is reserved for enabling/disablng IR remote control (0=>diabled, 1=>enabled)
// - fourth bit is reserved for sleep (1 => go to sleep for 1 minute)
// Right, Left: speed (in percentage); MSBit indicate direction: 1=forward, 0=backward; values from 0 to 100

#define OVERHEAD_SIZE 2*NUM_ROBOTS+1
// overhead content :
// - command: 1 byte, indicates which command the payload refer to
// - address: 2 bytes per robot


static struct libusb_device_handle *devh = NULL;

static int find_nrf_device(void)
{
	devh = libusb_open_device_with_vid_pid(NULL, 0x1915, 0x0101);
	return devh ? 0 : -1;
}

int usb_send(char* data, int nbytes) {

	int transferred = 0;
	int r = 0;

	r = libusb_bulk_transfer(devh, 0x01, data, 64, &transferred, 5000); // address 0x01
	if (r < 0) {
		fprintf(stderr, "bulk write error %d\n", r);
		return r;
	}
	if (transferred < nbytes) {
		fprintf(stderr, "short write (%d)\n", r);
		return -1;
	}

	return 0;

}

int usb_receive(char* data, int nbytes) {

	int received = 0;
	int r = 0;

	r = libusb_bulk_transfer(devh, 0x81, data, nbytes, &received, 5000);
	if (r < 0) {
		fprintf(stderr, "bulk read error %d\n", r);
		return r;
	}
	if (received < nbytes) {
		fprintf(stderr, "short read (%d)\n", r);
		return -1;
	}

    return 0;

}

int main(void) {

	int r;
	unsigned int i;
    char RX_buffer[64];         // Last packet received from base station
    char TX_buffer[64];         // Next packet to send to base station
    char lspeed=0, rspeed=0;

    SYSTEMTIME start, end;
    FILETIME startF, endF;
    ULONGLONG start64, end64;

	r = libusb_init(NULL);
	if (r < 0)
		return r;

	r = find_nrf_device();
	if (r < 0) {
		fprintf(stderr, "Could not find/open device\n");
	}

	r = libusb_claim_interface(devh, 0);
	if (r < 0) {
		fprintf(stderr, "usb_claim_interface error %d\n", r);
	}
	printf("claimed interface\n");



    //SEND_PACKET FRAME: CMD | PLOAD_LENGTH | PLOAD | ADDRESS
    TX_buffer[0]=0x28; //0x27 = SEND_ALL_PACKET;

    lspeed = 35;
    rspeed = 50;

    for(i=0; i<1000; i++) {
    //for(i=0; i<3; i++) {
    //while(1) {

        // first robot
        TX_buffer[(0*8)+1] = i%100;          // R
        TX_buffer[(0*8)+2] = i%100;          // B
        TX_buffer[(0*8)+3] = i%100;          // G
        //if(i>=50) {
        //    TX_buffer[(0*8)+4] = 0x08;       // go to sleep
        //} else {
            TX_buffer[(0*8)+4] = 0x04;       // activate IR remote control
        //}
        TX_buffer[(0*8)+5] = 0&0xFF | 0x80; // speed right (in percentage)
        TX_buffer[(0*8)+6] = 0&0xFF | 0x80;  // speed left (in percentage)
        TX_buffer[(0*8)+7] = (3082>>8)&0xFF; // address of the robot
        TX_buffer[(0*8)+8] = 3082&0xFF;

//        if(i>=50) { // goto sleep
//            TX_buffer[(0*8)+1] = 0;          // R
//            TX_buffer[(0*8)+2] = 0;          // B
//            TX_buffer[(0*8)+3] = 0;          // G
//            TX_buffer[(0*8)+4] = 0x18;       // go to sleep
//            TX_buffer[(0*8)+5] = 0;         // speed right (in percentage)
//            TX_buffer[(0*8)+6] = 0;         // speed left (in percentage)
//            TX_buffer[(0*8)+7] = (3041>>8)&0xFF; // address of the robot
//            TX_buffer[(0*8)+8] = 3041&0xFF;
//        }

        // second robot
        TX_buffer[(1*8)+1] = i%100;          // R
        TX_buffer[(1*8)+2] = 0;              // B
        TX_buffer[(1*8)+3] = 0;              // G
        if(i==100) {
            TX_buffer[(1*8)+4] = 0x08;       // sleep
        } else {
            TX_buffer[(1*8)+4] = 0x04;       // activate IR remote control
        }
        TX_buffer[(1*8)+5] = 0&0xFF | 0x80; // speed right (in percentage)
        TX_buffer[(1*8)+6] = 0&0xFF | 0x80;  // speed left (in percentage)
        TX_buffer[(1*8)+7] = (3085>>8)&0xFF; // address of the robot
        TX_buffer[(1*8)+8] = 3085&0xFF;

        // third robot
        TX_buffer[(2*8)+1] = 0;               // R
        TX_buffer[(2*8)+2] = 0;               // B
        TX_buffer[(2*8)+3] = i%100;           // G
        if(i==100) {
            TX_buffer[(2*8)+4] = 0x08;        // sleep
        } else {
            TX_buffer[(2*8)+4] = 0x04;        // activate IR remote control
        }
        TX_buffer[(2*8)+5] = 0&0xFF | 0x80;  // speed right (in percentage)
        TX_buffer[(2*8)+6] = 0&0xFF | 0x80;   // speed left (in percentage)
        TX_buffer[(2*8)+7] = (3101>>8)&0xFF;  // address of the robot
        TX_buffer[(2*8)+8] = 3101&0xFF;

        // fourth robot
        TX_buffer[(3*8)+1] = 0;               // R
        TX_buffer[(3*8)+2] = i%100;           // B
        TX_buffer[(3*8)+3] = 0;               // G
        TX_buffer[(3*8)+4] = 0x00;            // IR/flags
        TX_buffer[(3*8)+5] = 0&0xFF | 0x80;   // speed right (in percentage)
        TX_buffer[(3*8)+6] = 0&0xFF | 0x80;   // speed left (in percentage)
        TX_buffer[(3*8)+7] = (3024>>8)&0xFF;  // address of the robot
        TX_buffer[(3*8)+8] = 3024&0xFF;


        r = usb_send(TX_buffer, PACKETS_SIZE+OVERHEAD_SIZE);
        if(r < 0) {
            printf("send error!\n");
        }

        printf("bootloader entered\n");
        exit(0);

        RX_buffer[0] = 0;
        RX_buffer[1] = 0;
        RX_buffer[2] = 0;
        RX_buffer[3] = 0;
        r = usb_receive(RX_buffer, 64);
        if(r < 0) {
            printf("receive error!\n");
        }

        //printf("Battery level 1 = %d\n", (int)((unsigned char)RX_buffer[0]));
        //printf("Battery level 2 = %d\n", (int)((unsigned char)RX_buffer[1]));
        //printf("Battery level 3 = %d\n", (int)((unsigned char)RX_buffer[2]));
        //printf("Battery level 4 = %d\n", (int)((unsigned char)RX_buffer[3]));
        printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", (int)((unsigned char)RX_buffer[0]), (int)((unsigned char)RX_buffer[1]), (int)((unsigned char)RX_buffer[2])
               , (int)((unsigned char)RX_buffer[3]), (int)((unsigned char)RX_buffer[4]), (int)((unsigned char)RX_buffer[5]), (int)((unsigned char)RX_buffer[6])
               , (int)((unsigned char)RX_buffer[7]), (int)((unsigned char)RX_buffer[8]), (int)((unsigned char)RX_buffer[9]), (int)((unsigned char)RX_buffer[10])
               , (int)((unsigned char)RX_buffer[11]), (int)((unsigned char)RX_buffer[12]), (int)((unsigned char)RX_buffer[13]), (int)((unsigned char)RX_buffer[14])
               , (int)((unsigned char)RX_buffer[15]));

        printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", (int)((unsigned char)RX_buffer[16]), (int)((unsigned char)RX_buffer[17]), (int)((unsigned char)RX_buffer[18])
               , (int)((unsigned char)RX_buffer[19]), (int)((unsigned char)RX_buffer[20]), (int)((unsigned char)RX_buffer[21]), (int)((unsigned char)RX_buffer[22])
               , (int)((unsigned char)RX_buffer[23]), (int)((unsigned char)RX_buffer[24]), (int)((unsigned char)RX_buffer[25]), (int)((unsigned char)RX_buffer[26])
               , (int)((unsigned char)RX_buffer[27]), (int)((unsigned char)RX_buffer[28]), (int)((unsigned char)RX_buffer[29]), (int)((unsigned char)RX_buffer[30])
               , (int)((unsigned char)RX_buffer[31]));

        printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", (int)((unsigned char)RX_buffer[32]), (int)((unsigned char)RX_buffer[33]), (int)((unsigned char)RX_buffer[34])
               , (int)((unsigned char)RX_buffer[35]), (int)((unsigned char)RX_buffer[36]), (int)((unsigned char)RX_buffer[37]), (int)((unsigned char)RX_buffer[38])
               , (int)((unsigned char)RX_buffer[39]), (int)((unsigned char)RX_buffer[40]), (int)((unsigned char)RX_buffer[41]), (int)((unsigned char)RX_buffer[42])
               , (int)((unsigned char)RX_buffer[43]), (int)((unsigned char)RX_buffer[44]), (int)((unsigned char)RX_buffer[45]), (int)((unsigned char)RX_buffer[46])
               , (int)((unsigned char)RX_buffer[47]));

        printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", (int)((unsigned char)RX_buffer[48]), (int)((unsigned char)RX_buffer[49]), (int)((unsigned char)RX_buffer[50])
               , (int)((unsigned char)RX_buffer[51]), (int)((unsigned char)RX_buffer[52]), (int)((unsigned char)RX_buffer[53]), (int)((unsigned char)RX_buffer[54])
               , (int)((unsigned char)RX_buffer[55]), (int)((unsigned char)RX_buffer[56]), (int)((unsigned char)RX_buffer[57]), (int)((unsigned char)RX_buffer[58])
               , (int)((unsigned char)RX_buffer[59]), (int)((unsigned char)RX_buffer[60]), (int)((unsigned char)RX_buffer[61]), (int)((unsigned char)RX_buffer[62])
               , (int)((unsigned char)RX_buffer[63]));

        // send data at 10 Hz
        while(1) {
            GetSystemTime(&end);
            SystemTimeToFileTime(&end, &endF);

            // Copy the time into a quadword.
            start64 = (((ULONGLONG) startF.dwHighDateTime) << 32) + startF.dwLowDateTime;
            //start64 /=  10000000; //100 ns resolution
            end64 = (((ULONGLONG) endF.dwHighDateTime) << 32) + endF.dwLowDateTime;
            //end64 /=  10000000;

            if((end64-start64)/10000 >= 100) { // 100 ms
                break;
            }

        }

    }


	libusb_release_interface(devh, 0);
	libusb_close(devh);
	libusb_exit(NULL);

	return 0;

}

